use std::{path::Path, sync::LazyLock};

use color_eyre::eyre::{Result, bail};
use reqwest::{
    StatusCode,
    blocking::{Client, multipart::Form},
    header::AUTHORIZATION,
};

use crate::Config;

static CLIENT: LazyLock<Client> = LazyLock::new(Client::new);

#[derive(Debug, Clone, serde::Deserialize)]
pub struct ZiplineFolder {
    pub id: String,
    pub name: String,
}

#[derive(Debug, serde::Deserialize)]
pub struct ZiplineUploadResponse {
    pub files: Vec<ZiplineFile>,
}

#[derive(Debug, serde::Deserialize)]
pub struct ZiplineFile {
    pub id: String,
    pub url: String,
}

#[derive(Debug, serde::Deserialize)]
pub struct ZiplineFileInfo {
    pub thumbnail: Option<ZiplineThumbnail>,
}

#[derive(Debug, serde::Deserialize)]
pub struct ZiplineThumbnail {
    pub path: String,
}

impl ZiplineFileInfo {
    pub fn thumbnail_url(&self, config: &Config) -> Option<String> {
        self.thumbnail
            .as_ref()
            .map(|t| format!("{}raw/{}", config.fixed_url(), t.path))
    }
}

pub fn get_folders(config: &Config) -> Result<Vec<ZiplineFolder>> {
    let url = format!("{}api/user/folders?noincl=true", config.fixed_url());

    let res = CLIENT
        .get(url)
        .header(AUTHORIZATION, &config.zipline_token)
        .send()?;

    if res.status() != StatusCode::OK {
        bail!("an error occurred ({}): {}", res.status(), res.text()?);
    } else {
        res.json().map_err(Into::into)
    }
}

pub fn upload_file(
    config: &Config,
    folder: Option<&ZiplineFolder>,
    file_path: &Path,
) -> Result<ZiplineUploadResponse> {
    let url = format!("{}api/upload", config.fixed_url());

    let form = Form::new().file("file", file_path)?;

    let mut req = CLIENT
        .post(url)
        .header(AUTHORIZATION, &config.zipline_token)
        .header("x-zipline-format", "name")
        .multipart(form);

    if let Some(folder) = folder {
        req = req.header("x-zipline-folder", &folder.id);
    }

    let res = req.send()?;

    if res.status() != StatusCode::OK {
        bail!("an error occurred ({}): {}", res.status(), res.text()?);
    } else {
        res.json().map_err(Into::into)
    }
}

pub fn recalc_thumbnails(config: &Config) -> Result<()> {
    let url = format!("{}api/server/thumbnails", config.fixed_url());

    let res = CLIENT
        .post(url)
        .header(AUTHORIZATION, &config.zipline_token)
        .json(&[("rerun", false)])
        .send()?;

    if res.status() != StatusCode::OK {
        bail!("an error occurred ({}): {}", res.status(), res.text()?);
    } else {
        Ok(())
    }
}

pub fn get_file_details(config: &Config, id: &str) -> Result<ZiplineFileInfo> {
    let url = format!("{}api/user/files/{id}", config.fixed_url());

    let res = CLIENT
        .get(url)
        .header(AUTHORIZATION, &config.zipline_token)
        .send()?;

    if res.status() != StatusCode::OK {
        bail!("an error occurred ({}): {}", res.status(), res.text()?);
    } else {
        res.json().map_err(Into::into)
    }
}
